﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using WinForms = System.Windows.Forms;

namespace WpfApplication1
{
    /// <summary>
    /// Window1.xaml の相互作用ロジック
    /// </summary>
    public partial class Window1 : Window
    {
        ObservableCollection<Border> _thumbnails;        

        public Window1()
        {
            InitializeComponent();
            
            _thumbnails = new ObservableCollection<Border>();
            _listbox.ItemsSource = _thumbnails;
            _browser.Navigated += new NavigatedEventHandler(_browser_Navigated);
        }

        
        private void TextBox_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            { 
                case Key.Return:
                    try
                    {   
                        if(null == _browser.Source || ( _browser.Source.ToString() != ((sender as TextBox).Text)))
                            _browser.Source = new Uri((sender as TextBox).Text);
                    }
                    catch (Exception)
                    {
                        return;
                    }
                    break;
                default:
                    return;
            }
            e.Handled = true;
        }
        
        void addThumbnail()
        {
            //see http://blogs.msdn.com/jaimer/archive/2007/03/07/using-visualtreehelper-getdrawing-to-get-around-the-z-order-limitation-when-hosting-windows-forms-controls.aspx

            /*
             * in a delegate, web browser drawing is get as a thumbnail image.
             * the delegate must be called throw dispatcher with a low priority (maybe, less than DispatcherPriority.Render), otherwise while box (empty image) is drawn.
             */
            this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Background,
                           new System.Windows.Threading.DispatcherOperationCallback(delegate
                           {
                               DrawingGroup dg = VisualTreeHelper.GetDrawing(_browser);

                               DrawingBrush db = new DrawingBrush(dg);
                               db.TileMode = TileMode.None;
                               db.Stretch = Stretch.Uniform;
                               Rectangle rect = new Rectangle();
                               rect.Width  = _browser.ActualHeight;
                               rect.Height = _browser.ActualWidth;
                               rect.Fill = db;
                               Border brd = new Border();
                               brd.Child = rect;
                               _thumbnails.Add(brd);


                               return null;

                           }), null);          
            
            // WPF UIElement (that inheritates Visual) can be drawn as an image using VisualBrush.
            /* 
             * This sample code convertes TextBox as image. 
             */
            /*
            this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Background,
                new System.Windows.Threading.DispatcherOperationCallback(delegate
                {
                    VisualBrush vb = new VisualBrush(_textbox);
                    //vb.ViewboxUnits = BrushMappingMode.Absolute;
                    //vb.Viewbox = new Rect(new Size(_browser.Width, _browser.Height));
                    Rectangle rect = new Rectangle();
                    rect.Width     = _textbox.ActualWidth;
                    rect.Height    = _textbox.ActualHeight;
                    rect.Fill = vb;
                    Border brd = new Border();
                    brd.Child = rect;
                    _thumbnails.Add(brd);
                    
                    return null;

                }), null);
            */

            /* 
             * The web browser can not be converted by this method, because WebBrowser class is an IE wrapper, not a native wpf control.
             */ 
            /* 
            this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Background,
                new System.Windows.Threading.DispatcherOperationCallback(delegate
                {
                    VisualBrush vb = new VisualBrush(_browser);
                    //vb.ViewboxUnits = BrushMappingMode.Absolute;
                    //vb.Viewbox = new Rect(new Size(_browser.Width, _browser.Height));
                    Rectangle rect = new Rectangle();
                    rect.Width = _browser.Width;
                    rect.Height = _browser.Height;
                    rect.Fill = vb;
                    Border brd = new Border();
                    brd.Child = rect;
                    _thumbnails.Add(brd);

                    return null;

                }), null);
            */
        }

        private void _browser_Navigated(object sender, NavigationEventArgs e)
        {
            string uri = _browser.Source.ToString();
            if (_textbox.Text != uri)
                _textbox.Text = uri;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            addThumbnail();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            _thumbnails.Clear();
        }      
    }
}
